indexing
   title:       "TIMESTAMP"
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    22,Sep,95
   version:     1.0
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "timestam.e";

class TIMESTAMP

   inherit
      DATE
         undefine
            set_to_now,
            clear,
            difference,
            infix "<",
            infix "+",
            plus,
            infix "-",
            minus
         end;

      TIME
         undefine
            set_to_now,
            clear,
            difference,
            infix "<",
            infix "+",
            plus,
            infix "-",
            minus
         end;

   creation
      clear,
      set,
      set_from_compressed,
      set_to_now


   feature

      set( yr, mh, dy, hr, me : INTEGER; sd : REAL ) is
         require else
            date_time_valid : tt.valid_date( yr, mh, dy ) and then
                              tt.valid_time( hr, me, sd )

         do
            set_date( yr, mh, dy );
            set_time( hr, me, sd );

         end -- set

      set_from_compressed( abs_day, millsec : INTEGER ) is
         -- if necessary with normalisation
         local
            ms, dy : INTEGER;
         do
            ms := millsec;
            dy := abs_day;
            if ms >= tt.milliseconds_in_day then
               dy := dy + ms // tt.milliseconds_in_day;
               ms := ms \\ tt.milliseconds_in_day;
            elseif ms < tt.zero then
               dy := dy + ms // tt.milliseconds_in_day - tt.one_day;
               ms := ms \\ tt.milliseconds_in_day + tt.one_day;
            end;

            set_from_absolute_days ( dy );
            set_from_milliseconds ( ms );

         end -- set_from_compressed

      set_to_now is
         local
            now : NOW
         do
            !!now;
            set( now.year - tt.one_year - tt.one_year, now.month, now.day,
               now.hour, now.minute, now.second );
         end -- set_to_now

      clear is
         do
            milliseconds  := tt.zero;
            absolute_days := tt.zero;
         end -- clear

      infix "<"( ts : like Current ) :  BOOLEAN is
         do
            Result :=
               ( Current.absolute_days <= ts.absolute_days ) and then
               ( Current.milliseconds  <  ts.milliseconds  )
         end -- infix "<"

      infix "+",
      plus( d: DURATION ) :  TIMESTAMP is
         require else
            date_time_valid : (milliseconds + d.milliseconds) // tt.milliseconds_in_day +
                              absolute_days + d.absolute_days  > tt.zero
         local
            abs_day, millsec : INTEGER;
         do
            abs_day := absolute_days + d.absolute_days;
            millsec := milliseconds + d.milliseconds;

            -- adjust carry
            if millsec < tt.zero then
               millsec := millsec + tt.milliseconds_in_day;
               abs_day := abs_day - tt.one_day;
            end -- if

            if millsec >= tt.milliseconds_in_day then
               millsec := millsec - tt.milliseconds_in_day;
               abs_day := abs_day + tt.one_day;
            end -- if

            !!Result.set_from_compressed( abs_day, millsec );

         end -- infix "+", plus

      infix "-",
      minus( d: DURATION ) :  TIMESTAMP is
         require else
            date_time_valid : (milliseconds - d.milliseconds) //
               tt.milliseconds_in_day + absolute_days - d.absolute_days  > tt.zero
         local
            abs_day, millsec : INTEGER;
         do
            abs_day := absolute_days - d.absolute_days;
            millsec := milliseconds - d.milliseconds;

            -- adjust carry
            if millsec < tt.zero then
               millsec := millsec + tt.milliseconds_in_day;
               abs_day := abs_day - tt.one_day;
            end -- if

            if millsec >= tt.milliseconds_in_day then
               millsec := millsec - tt.milliseconds_in_day;
               abs_day := abs_day + tt.one_day;
            end -- if

            !!Result.set_from_compressed( abs_day, millsec );

         end -- infix "-", minus

      difference ( t :  like Current ) :  DURATION is
         local
            dy, me : INTEGER;
         do
            dy := absolute_days - t.absolute_days;
            me := milliseconds - t.milliseconds;
            if me < tt.zero then
               me := me + tt.milliseconds_in_day;
               dy := dy - tt.one_day;
            end
            !!Result.set_from_compressed ( dy, me );
         end -- difference

end -- class TIMESTAMP
